use std::sync::{Arc,RwLock};
use std::time::Duration;

use pfs_time_measure::Clock;

const NUM_THREADS:usize = 16;


fn sleep_millis(millis: u64) {
    std::thread::sleep(Duration::from_millis(millis));
}

fn main() {

    let rwlocked = Arc::new(RwLock::new(42));

    let mut threads = Vec::with_capacity(NUM_THREADS);

    for x in 0..NUM_THREADS {

        let r = rwlocked.clone();

        let t = std::thread::spawn(move|| {
            let mut clock = Clock::new(format!("t{x} reader_lock"), false);
            loop {
                {
                    clock.restart();
                    let rr = r.read().unwrap();
                    clock.stop();
                    clock.print_all_measurements();
                    println!("[READER] acquired lock");
                    println!("[READER] t{x}: {}!", rr);
                    sleep_millis(500);
                    println!("[READER] released lock");
                }
                sleep_millis(100);
            }
        });
        threads.push(t);
    }

    let r = rwlocked.clone();
    let twrite = std::thread::spawn(move|| {
        sleep_millis(100);
        let mut clock = Clock::new("writer_lock".to_string(), false);
        loop {
            println!("[WRITER] trying to acquire lock");
            clock.restart();
            match r.write() {
                Ok(mut val) => {
                    clock.stop();
                    clock.print_all_measurements();
                    println!("[WRITER] acquired lock");
                    *val = 11;
                    sleep_millis(600);
                },
                Err(e) => {
                    clock.stop();
                    clock.print_all_measurements();
                    println!("[WRITER] error: {:?}", e);
                },
            }
            println!("[WRITER] released lock");
            sleep_millis(100);
        }
    });

    for t in threads {
        let _ = t.join();
    }

    let _ = twrite.join();
}
